//Algorithm to implement Classic2 Classification Method
//Bradley Turnbull, NCSU 

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <R.h>
#include <Rmath.h>

//make the sign function to be used later
int sgn( int value){
	if( value < 0 ){
		return -1; 
	}else if(value == 0 ){
		return 0;
	}else if(value > 0 ){
		return 1; 
	}
}

//function for hinge loss
double plus( double value){
	if( value < 0.0){
		return 0.0;
	}else{
		return value;
	}
}

//functions to count negatives and positives
int numNeg( double array[], int size ){
	int num = 0; 
	for(int i=0; i < size; i++){
		if( array[i] < 0.0 ){
			num++; 
		}
	}
	return num;
}

int numPos( double array[], int size ){
	int num = 0; 
	for(int i=0; i < size; i++){
		if( array[i] > 0.0 ){
			num++; 
		}
	}
	return num;
}

//Function to calculate the root of quadratic
double crossz( double x1, double y1, double x2, double y2){
	double root = x1 - (y1/( (y2-y1)/(x2-x1) ));
	return root; 
}

//Functions to compare values for qsort
int cmp_desc (const void *a, const void *b){
    const double *da = (const double *) a;
	const double *db = (const double *) b;
	return (*da < *db) - (*da > *db);
}

int cmp_ascd (const void *a, const void *b){
    const double *da = (const double *) a;
	const double *db = (const double *) b;
	return (*da > *db) - (*da < *db);
}

// function to find location of minimum element in an array 
int minLoc( double array[], int size){
	int loc;
	int min = 0; 
	
	for( loc = 0; loc < size; loc++){
		if( array[loc] < array[min] ){
			min = loc;	
		}
	}
	return min;
}


//The Important True Function
void Classic2( double *Y, double *initData, int *n, int *p, double *lambda1,
					double *lambda2, double *epsilon, int *M, int *correct,
					double *beta0 ,double *betas, int *keep, int *numKept ){
	//betas vector is all zeros to start


	//Set the data matrix (2D Array) from the initData
	int k=0; 
	double data[*n][*p];
	for( int j=0; j < *p; j++){
		for( int i=0; i < *n; i++){
			data[i][j] = initData[k];
			k++;
		}
	}

	//initial values for N+ and N-
	int N_plus = 0;
	int N_minus = 0;
	
	for(int i=0; i < *n; i++){
		if( Y[i] == 1){ 
			N_plus++;
		}else if( Y[i] == -1){
			N_minus++;
		}
	}
	
	//set the first beta_0 estimate
	//*beta0 = (double)( N_minus - N_plus) / *n;
	//Flipped to imitate Helen's code
	*beta0 = (double)( N_plus - N_minus) / *n;
	//Rprintf("beta0 = %f \n", *beta0); 
	//*beta0= 0.0;   
	  
	//Vector of residuals
	double S[*M+1];
	S[0] = 0.0; 
	for(int i=0; i < *n; i++){
		S[0] = S[0] + pow(plus(1 - Y[i]* *beta0),2); 
		//S[0] = S[0] + pow( plus(1.0), 2 );
	}
	//S[0] = 1.0; 
	
	//the matrix c that is n by M
	double c[*n][*M+1];
	  
	for(int i=0; i<*n; i++){
		c[i][0] = 1 - Y[i]* *beta0;
		//c[i][0] = 1.0; 
	}
	
	//Counting parameters for algorithm
	double F_plus, F_minus, T1_0, T2_0, T3_0, T4_0, 
									T5_0,T6_0,T7_0,T8_0; 

	//vector of current beta estimates, changes with each loop
	double beta_hat[*p];	

//Now Start the loop to select the variables
int m; //declare m outside for loop so scope is outside loop  
for(m=0; m < *M; m++){
	for( int j=0; j < *p; j++){
		
		//compute the T_0 values;
		T1_0=T2_0=T3_0=T4_0=T5_0=T6_0=T7_0=T8_0=0.0; 
		
		for(int i=0; i<*n; i++){
			if(Y[i]==1 && data[i][j]>0 && c[i][m]/data[i][j]>=0){
				T1_0 = T1_0 + c[i][m] * data[i][j];
				T2_0 = T2_0 + pow( data[i][j], 2);  
			}else if( Y[i]==1 && data[i][j]<0 && c[i][m]/data[i][j]<=0){
				T5_0 = T5_0 + c[i][m] * data[i][j];
				T6_0 = T6_0 + pow( data[i][j], 2); 
			}else if( Y[i]==-1 && data[i][j]>0 && c[i][m]/data[i][j]>=0){
				T3_0 = T3_0 + c[i][m] * data[i][j]; 
				T4_0 = T4_0 + pow( data[i][j], 2);
			}else if( Y[i]==-1 && data[i][j]<0 && c[i][m]/data[i][j]<=0){
				T7_0 = T7_0 + c[i][m] * data[i][j];
				T8_0 = T8_0 + pow( data[i][j], 2); 
			}
		}
		
		//calculate the sum of the T_0's
		double sumT_0 = fabs( -T1_0 + T3_0 - T5_0 + T7_0 ); 
		
		//see if sumT_0 is less than lambda1, if so set beta.hat[j] to 0 and exit
		if( sumT_0 < *lambda1 / 2){
			beta_hat[j] = 0.0;
			
		}else{ 
		//since not less than lambda1 continue on with algorithm 
			//calculate F.plus and F.minus
			F_plus = 2*(-T1_0 + T3_0 - T5_0 + T7_0) + *lambda1;
			F_minus = F_plus - 2* *lambda1;
			//Rprintf( "F0+ = %f,  F0- = %f \n", F_plus, F_minus );
	
			//see which side of zero they are on
			if( F_minus > 0.0 ){
			  
			  //array to hold values of cY/X vector
			  double cYX[*n]; 
			  for(int i=0; i < *n; i++){
				cYX[i] = c[i][m]*Y[i]/data[i][j];
				//Rprintf( "%f \t", cYX[i] ); 
			  }
				
			  //Code to get knot vector
			  
			  //count how many negative values
			  //add 1 to include first value = 0 
			  int num_neg = numNeg(cYX, *n) + 1; 
			
			  //make vector of just negative values, first value is 0
			  double negVec[num_neg];  
			  int count=1; 
			  negVec[0] = 0.0; 
			  for( int i = 0; i < *n; i++){
				if(cYX[i] < 0.0){
				  negVec[count] = cYX[i];
				  count++;
				}
			  }
			
			  //sort the negVec vector descending
			  qsort( negVec, num_neg, sizeof(double), cmp_desc);
			  
			  //get only the unique values
			  int numUnique=0; double curVal=0.0;
			  int is_unique[num_neg];
			  is_unique[0] = numUnique = 1; //set initial values 
			  for( int i=1; i<num_neg; i++){
			 	if( fabs( negVec[i] - curVal ) > 1e-10 ){
				  is_unique[i] = 1; //set to 1 if unique
				  numUnique++;
				  curVal = negVec[i];  
				}else{
				  is_unique[i] = 0; //set to 0 if not
				}
			  }
			  //make vector of just unique values
			  double xi[numUnique];
			  count=0;
			  for( int i=0; i<num_neg; i++){
				if( is_unique[i] == 1){
				  xi[count] = negVec[i];
				  count++;
				  //Rprintf("%f \t", xi[count-1]);
				}
			  }
			  //Rprintf( "\n \n"); 
			  
			  //set first component of T's
			  double oldF, newF; 
			  double oldTodd, oldTeven, newTodd, newTeven;  
			  //Create all the T_k values
			  double T1, T2, T3, T4, T5, T6, T7, T8; 
			  T1=T2=T3=T4=T5=T6=T7=T8=0.0; 
			  //Handle the first knot separately, don't worry about 0
			  int l=1; 
			  for( int i=0; i<*n; i++){
				if(Y[i]==1 && data[i][j]>0 && c[i][m]/data[i][j]>=xi[l] ){
				  T1 = T1 + c[i][m]*data[i][j];
				  T2 = T2 + pow( data[i][j], 2);
				}
				if(Y[i]==1 && data[i][j]<0 && c[i][m]/data[i][j]<=xi[l] ){
				  T3 = T3 + c[i][m]*data[i][j];
				  T4 = T4 + pow( data[i][j], 2);
				}
				if(Y[i]==-1 && data[i][j]>0 && c[i][m]/data[i][j]>= -xi[l] ){
				  T5 = T5 + c[i][m]*data[i][j];
				  T6 = T6 + pow( data[i][j], 2);
				}
			  	if(Y[i]==-1 && data[i][j]<0 && c[i][m]/data[i][j]<= -xi[l] ){
				  T7 = T7 + c[i][m]*data[i][j];
				  T8 = T8 + pow( data[i][j], 2);
				}
			  }
			  
			  //calc values
			  oldTodd = newTodd = (-T1-T3+T5+T7);
			  oldTeven = newTeven = (T2+T4+T6+T8);
			  newF = 2*( oldTodd + xi[l]*oldTeven + *lambda2*xi[l] ) - *lambda1; 
			  //Rprintf("First Fprime = %f \n", newF);
			  
			  if( newF==0.0 ){
			  	beta_hat[j] = xi[l]; 
			  }else if( newF < 0.0 ){
			  	beta_hat[j] = crossz( 0, F_minus, xi[l], newF);
			  }else{
			  	//search for optimal knot
			  
			  	//Rprintf("Negative: \n");
			  	while( newF > 0.0 ){
				  l++; //increase l
				  oldF = newF;
				  
				  for( int i=0; i<*n; i++){
				  	if(Y[i]==1 && data[i][j]>0 && c[i][m]/data[i][j]== xi[l] ){
					  newTodd = newTodd - c[i][m]*data[i][j]; 
				  	  newTeven = newTeven + pow( data[i][j], 2);
				  	}	
				  	if(Y[i]==1 && data[i][j]<0 && c[i][m]/data[i][j]== xi[l-1] ){
					  newTodd = newTodd + c[i][m]*data[i][j];
					  newTeven = newTeven - pow( data[i][j], 2); 
				    }	
					if(Y[i]==-1 && data[i][j]>0 && c[i][m]/data[i][j]== -xi[l-1] ){
					  newTodd = newTodd - c[i][m]*data[i][j]; 
				      newTeven = newTeven - pow( data[i][j], 2); 
				  	}   
					if(Y[i]==-1 && data[i][j]<0 && c[i][m]/data[i][j]== -xi[l] ){
					  newTodd = newTodd + c[i][m]*data[i][j];
					  newTeven = newTeven + pow( data[i][j], 2); 
				  	} 
				  }
								
				  //Calculate F
				  newF = 2*(newTodd + xi[l]*newTeven + *lambda2*xi[l]) - *lambda1;
				  //Rprintf("nTodd=%f;  nTeven=%f;  newF=%f \n", newTodd, newTeven,newF);
				   //update
				  oldTodd = newTodd;
				  oldTeven = newTeven;
			  	}
				
				if( newF == 0.0){
				  beta_hat[j] = xi[l];
				}else if( newF < 0.0){
				  beta_hat[j] = crossz( xi[l-1],oldF,xi[l],newF ); 
				}
			  }	
			}else if( F_plus < 0.0 ){
			  //array to hold values of cY/X vector
			  double cYX[*n]; 
			  for(int i=0; i < *n; i++){
			   	cYX[i] = c[i][m]*Y[i]/data[i][j]; 
			  }
			  //count how many positive values
			  //add 1 to include first value = 0 
			  int num_pos = numPos(cYX, *n)+1; 
			
			  //make vector of just positive values, first value is 0
			  double posVec[num_pos];  
			  posVec[0] = 0.0;
			  int count=1;  
			  for( int i = 0; i < *n; i++){
				if(cYX[i] > 0.0){
				  posVec[count] = cYX[i];
				  count++; 
				}	
			  }
			
			  //sort the posVec vector ascending
			  qsort( posVec, num_pos, sizeof(double), cmp_ascd);
			  			
			  //get only the unique values
			  int numUnique=0; double curVal=0.0;
			  int is_unique[num_pos];
			  is_unique[0] = numUnique = 1; //set initial values 
			  for( int i=1; i<num_pos; i++){
			 	if( fabs( posVec[i] - curVal ) > 1e-10 ){
				  is_unique[i] = 1; //set to 1 if unique
				  numUnique++;
				  curVal = posVec[i];  
				}else{
				  is_unique[i] = 0; //set to 0 if not
				}
			  }
			  //make vector of just unique values
			  double xi[numUnique];
			  count=0;
			  //Rprintf("Positive xi's: "); 
			  for( int i=0; i<num_pos; i++){
				if( is_unique[i] == 1){
				  xi[count] = posVec[i];
				  count++;
				  //Rprintf("%f \t", xi[count-1]);
				}
			  }
			  //Rprintf( "\n \n"); 
			  
			  //set first component of T's
			  double oldF, newF; 
			  double oldTodd, oldTeven, newTodd, newTeven;  
			  //Create all the T_k values
			  double T1, T2, T3, T4, T5, T6, T7, T8; 
			  T1=T2=T3=T4=T5=T6=T7=T8=0.0; 
			  //Handle the first knot separately, don't worry about 0
			  int l=1; 
			  for( int i=0; i<*n; i++){
				if(Y[i]==1 && data[i][j]>0 && c[i][m]/data[i][j]>=xi[l] ){
				  T1 = T1 + c[i][m]*data[i][j];
				  T2 = T2 + pow( data[i][j], 2);
				}
				if(Y[i]==1 && data[i][j]<0 && c[i][m]/data[i][j]<=xi[l] ){
				  T3 = T3 + c[i][m]*data[i][j];
				  T4 = T4 + pow( data[i][j], 2);
				}
				if(Y[i]==-1 && data[i][j]>0 && c[i][m]/data[i][j]>= -xi[l] ){
				  T5 = T5 + c[i][m]*data[i][j];
				  T6 = T6 + pow( data[i][j], 2);
				}
			  	if(Y[i]==-1 && data[i][j]<0 && c[i][m]/data[i][j]<= -xi[l] ){
				  T7 = T7 + c[i][m]*data[i][j];
				  T8 = T8 + pow( data[i][j], 2);
				}
			  }
			  
			  //calc values
			  oldTodd = newTodd = (-T1-T3+T5+T7);
			  oldTeven = newTeven = (T2+T4+T6+T8);
			  newF = 2*( oldTodd + xi[l]*oldTeven + *lambda2*xi[l] ) + *lambda1; 
			  //Rprintf("First Fprime = %f \n", newF);
			  
			  if( newF==0.0 ){
			  	beta_hat[j] = xi[l]; 
			  }else if( newF > 0.0 ){
			  	beta_hat[j] = crossz( 0, F_minus, xi[l], newF);
			  }else{
			  	//search for optimal knot
			  
			  	//Rprintf("Negative: \n");
			  	while( newF < 0.0 ){
				  l++; //increase l
				  oldF = newF;
				  
				  for( int i=0; i<*n; i++){
				  	if(Y[i]==1 && data[i][j]>0 && c[i][m]/data[i][j]== xi[l-1] ){
					  newTodd = newTodd + c[i][m]*data[i][j]; 
				  	  newTeven = newTeven - pow( data[i][j], 2);
				  	}	
				  	if(Y[i]==1 && data[i][j]<0 && c[i][m]/data[i][j]== xi[l] ){
					  newTodd = newTodd - c[i][m]*data[i][j];
					  newTeven = newTeven + pow( data[i][j], 2); 
				    }	
					if(Y[i]==-1 && data[i][j]>0 && c[i][m]/data[i][j]== -xi[l] ){
					  newTodd = newTodd + c[i][m]*data[i][j]; 
				      newTeven = newTeven + pow( data[i][j], 2); 
				  	}   
					if(Y[i]==-1 && data[i][j]<0 && c[i][m]/data[i][j]== -xi[l-1] ){
					  newTodd = newTodd - c[i][m]*data[i][j];
					  newTeven = newTeven - pow( data[i][j], 2); 
				  	} 
				  }
				
				  //Calculate F
				  newF = 2*(newTodd + xi[l]*newTeven + *lambda2*xi[l]) + *lambda1;
				  //Rprintf("nTodd=%f;  nTeven=%f;  newF=%f \n", newTodd, newTeven,newF);
				  
				  //update
				  oldTodd = newTodd;
				  oldTeven = newTeven;
			  	}
				
				if( newF == 0.0){
				  beta_hat[j] = xi[l];
				}else if( newF > 0.0){
				  beta_hat[j] = crossz( xi[l-1],oldF,xi[l],newF ); 
				}
			  }	
			}
		}	
	}
	//Rprintf("Got here: find min \n");	
	//Determine which is the best predictor
	double resids[*p], dummysum; 
	for( int j1 = 0; j1 < *p; j1++){
		dummysum = 0.0;
		for( int i=0; i<*n; i++){
		  dummysum = dummysum+pow(plus(c[i][m]-Y[i]*data[i][j1]*beta_hat[j1]),2);
		}
		resids[j1] = dummysum;
	}
	int bestj = minLoc(resids, *p); 
	
	//store the best predictor value
	keep[m] = bestj; 
	//betas[bestj] = betas[bestj] + beta_hat[bestj];
	betas[m] = beta_hat[bestj];
	S[m+1] = resids[bestj];
	
	//update c values
	for( int i=0; i < *n; i++){
		c[i][m+1] = c[i][m] - Y[i]*data[i][bestj]*beta_hat[bestj];
		//Rprintf("%f    ", c[i][m+1]);
	}
	//Rprintf("\n m+1=%d \n",m+1);
	
	//see if we should exit the loop
	if( S[m] - S[m+1] < *epsilon){ 
		m = m+1; 
		break; 
	}
}
//Done with loop so say how many predictors were kept
*numKept = m; 
//Rprintf("\n m=%d \n",m);

//Do a correction for beta0 if asked to 
if(*correct == 1){
  //calculate necessary values
  double J1,J2,J3,J4,J5,J6,J7,J8; 
  J1=J2=J3=J4=J5=J6=J7=J8=0.0;
  //Rprintf("\n m=%d \n",m);
  for(int i=0; i<*n; i++){
	//Rprintf("%f    ", c[i][m] );
    if(Y[i]==1 && c[i][m]>=0){
	  J1 = J1 + c[i][m];
	  J2++; 
	}else if( Y[i]==-1 && c[i][m]>=0){
	  J3 = J3 + c[i][m]; 
	  J4++;
	}
  }
  //Calculate F(b)= 2(-J1+J2) + 2b(J3+J4)
  double Fb = 2*(-J1+J3);
  //Rprintf("\n Fb = %f \n", Fb);
  
  //if J1=J3, don't do anything because Fb=0
  
  //if Fb != 0
  if(J1 != J3){
  	if(Fb > 0.0){
	  //Rprintf("Got here 4: Fb = %f \n \n", Fb);
	  //array to hold values of cY vector
	  double cY[*n]; 
	  for(int i=0; i < *n; i++){
	    cY[i] = c[i][m]*Y[i]; 
	  }
				
	  //Code to get knot vector
			  
	  //count how many negative values
	  //add 1 to include first value = 0 
	  int num_neg = numNeg(cY, *n)+1; 
			
	  //make vector of just negative values, first value is 0
	  double negVec[num_neg];  
	  negVec[0] = 0.0;
	  int k=1;  
	  for( int i = 0; i < *n; i++){
		if(cY[i] < 0.0){
		  negVec[k] = cY[i];
		  k++;
		}	
	  }
			
	  //sort the negVec vector descending
	  qsort( negVec, num_neg, sizeof(double), cmp_desc); 
		
	  //get only the unique values
	  int numUnique; double curVal=0.0;
	  int is_unique[num_neg];
	  is_unique[0] = numUnique = 1; //set initial values 
	  for( int i=1; i<num_neg; i++){
		if( fabs(negVec[i] - curVal) > 1e-10 ){
		  is_unique[i] = 1; //set to 1 if unique
		  numUnique++; 
		  curVal = negVec[i]; 
		}else{
		  is_unique[i] = 0; //set to 0 if not
		}
	  }
	  //make vector of just unique values
	  double xi[numUnique];
	  k=0; 
	  //Rprintf( "xi vector: \t"); 
	  for( int i=0; i<num_neg; i++){
	    if( is_unique[i] == 1){
		  xi[k] = negVec[i];
		  k++;
		  //Rprintf( "%f \t", xi[i]);  
		}
	  }
	  //Rprintf("\n \n");
		
	   
	  double oldF, newF; 
	  double oldTodd, oldTeven, newTodd, newTeven;  
	  //Create all the T_k values
	  double T1, T2, T3, T4, T5, T6, T7, T8; 
	  T1=T2=T3=T4=T5=T6=T7=T8=0.0; 
	  //Handle the first knot separately, don't worry about 0
	  int l=1; 
	  for( int i=0; i<*n; i++){
		if(Y[i]==1 &&  c[i][m]>=xi[l] ){
			T1 = T1 + c[i][m];
			T2++;
		}
		if(Y[i]==-1 && c[i][m]>= -xi[l] ){
			T5 = T5 + c[i][m];
			T6++;
		}
	  }
			  
	  //calc values
	  oldTodd = newTodd = (-T1-T3+T5+T7);
	  oldTeven = newTeven = (T2+T4+T6+T8);
	  newF = 2*( oldTodd + xi[l]*oldTeven ); 
	  //Rprintf("First Fprime = %f \n", newF);
			  
	  if( newF==0.0 ){
	  	*beta0 = *beta0 + xi[l];
	  }else if( newF < 0.0 ){
		*beta0 = *beta0 + crossz( 0, Fb, xi[l], newF);
	  }else{
	  //search for optimal knot
			  
	  while( newF > 0.0 ){
		l++; //increase l
		oldF = newF;
				  
		for( int i=0; i<*n; i++){
		  if(Y[i]==1  && c[i][m]== xi[l] ){
			newTodd = newTodd - c[i][m]; 
			newTeven = newTeven + 1;
		  }	
		  if(Y[i]==-1 &&  c[i][m]== -xi[l-1] ){
			newTodd = newTodd - c[i][m]; 
			newTeven = newTeven - 1; 
		  }   
		}
								
		//Calculate F
	    newF = 2*(newTodd + xi[l]*newTeven + *lambda2*xi[l]) - *lambda1;
		//Rprintf("nTodd=%f;  nTeven=%f;  newF=%f \n", newTodd, newTeven,newF);
		//update
		oldTodd = newTodd;
		oldTeven = newTeven;
	  }
				
	  if( newF == 0.0){
		*beta0 = *beta0 + xi[l];
	  }else if( newF < 0.0){
		*beta0 = *beta0 + crossz( xi[l-1],oldF,xi[l],newF );
		//Rprintf("add to beta0, %f, \n",  crossz( xi[l-1],oldF,xi[l],newF ));
	  }
	}	
	
	}else if( Fb < 0.0 ){
	  //Rprintf("Got here 5: Fb = %f \n", Fb );
	  //array to hold values of cY vector
	  double cY[*n]; 
	  for(int i=0; i < *n; i++){
		cY[i] = c[i][m]*Y[i]; 
	  }
	  //count how many positive values
	  //add 1 to include first value = 0 
	  int num_pos = numPos(cY, *n)+1; 
			
	  //make vector of just positive values, first value is 0
	  double posVec[num_pos];  
	  posVec[0] = 0; 
	  int count=1; 
	  for( int i = 0; i < *n; i++){
		if(cY[i] > 0){
		posVec[count] = cY[i];
		count++; 
		}	
	  }
			
	  //sort the posVec vector ascending
	  qsort( posVec, num_pos, sizeof(double), cmp_ascd); 
		
	  //get only the unique values
	  int numUnique=0; double curVal=0.0;
	  int is_unique[num_pos];
	  is_unique[0] = numUnique = 1; //set initial values 
	  for( int i=1; i<num_pos; i++){
		if( fabs(posVec[i] - curVal) > 1e-10 ){
		  is_unique[i] = 1; //set to 1 if unique
		  numUnique++; 
		  curVal = posVec[i];
	    }else{
		  is_unique[i] = 0; //set to 0 if not
		}
	  }
	  //make vector of just unique values
	  double xi[numUnique];
	  count = 0; 
	  for( int i=0; i<num_pos; i++){
		if( is_unique[i] == 1){
		  xi[count] = posVec[i];
		  count++; 
		}
	  }
	  
	  //set first component of T's
	  double oldF, newF; 
	  double oldTodd, oldTeven, newTodd, newTeven;  
	  //Create all the T_k values
	  double T1, T2, T3, T4, T5, T6, T7, T8; 
	  T1=T2=T3=T4=T5=T6=T7=T8=0.0; 
	  //Handle the first knot separately, don't worry about 0
	  int l=1; 
	  for( int i=0; i<*n; i++){
		if(Y[i]==1 &&  c[i][m]>=xi[l] ){
		  T1 = T1 + c[i][m];
		  T2 = T2 + 1;
		}
		if(Y[i]==-1 && c[i][m]>= -xi[l] ){
		  T5 = T5 + c[i][m];
		  T6 = T6 + 1;
		}
	  }
			  
	  //calc values
	  oldTodd = newTodd = (-T1-T3+T5+T7);
	  oldTeven = newTeven = (T2+T4+T6+T8);
	  newF = 2*( oldTodd + xi[l]*oldTeven ); 
	  //Rprintf("First Fprime = %f \n", newF);
			  
	  if( newF==0.0 ){
		*beta0 = *beta0 + xi[l]; 
	  }else if( newF > 0.0 ){
		*beta0 = *beta0 + crossz( 0, Fb, xi[l], newF);
	  }else{
				
	  //Rprintf("Negative: \n");
	  while( newF < 0.0 ){
		l++; //increase l
		oldF = newF;
				  
		for( int i=0; i<*n; i++){
		  if(Y[i]==1 &&  c[i][m]== xi[l-1] ){
			newTodd = newTodd + c[i][m]; 
			newTeven = newTeven - 1;
		  }	
		  if(Y[i]==-1 && c[i][m]== -xi[l] ){
			newTodd = newTodd + c[i][m]; 
			newTeven = newTeven + 1; 
		  }   
		}
				
		//Calculate F
		newF = 2*(newTodd + xi[l]*newTeven );
		//Rprintf("nTodd=%f;  nTeven=%f;  newF=%f \n", newTodd, newTeven,newF);
				  
		//update
		oldTodd = newTodd;
		oldTeven = newTeven;
	  }
		
	  if( newF == 0.0){
		*beta0 = *beta0 + xi[l];
	  }else if( newF > 0.0){
		*beta0 = *beta0 + crossz( xi[l-1],oldF,xi[l],newF ); 
		//Rprintf("add to beta0, %f, \n",  crossz( xi[l-1],oldF,xi[l],newF ));
	  }
  	}		
  }
}
}
}

	