//Algorithm to implement Classic1 Classification Method
//Bradley Turnbull, NCSU 

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <R.h>
#include <Rmath.h>

//make the sign function to be used later
double sgn( int value){
	if( value < 0 ){
		return -1.0; 
	}else if(value == 0 ){
		return 0.0;
	}else if(value > 0 ){
		return 1.0; 
	}
}

//function for hinge loss
double plus( double value){
	if( value < 0.0){
		return 0.0;
	}else{
		return value;
	}
}

//functions to count negatives and positives
int numNeg( double array[], int size ){
	int num = 0; 
	for(int i=0; i < size; i++){
		if( array[i] < 0.0 ){
			num++; 
		}
	}
	return num;
}

int numPos( double array[], int size ){
	int num = 0; 
	for(int i=0; i < size; i++){
		if( array[i] > 0.0 ){
			num++; 
		}
	}
	return num;
}


//Functions to compare values for qsort
int cmp_desc (const void *a, const void *b){
    const double *da = (const double *) a;
	const double *db = (const double *) b;
	return (*da < *db) - (*da > *db);
}

int cmp_ascd (const void *a, const void *b){
    const double *da = (const double *) a;
	const double *db = (const double *) b;
	return (*da > *db) - (*da < *db);
}

// function to find location of minimum element in an array 
int minLoc( double array[], int size){
	int loc;
	int min = 0; 
	
	for( loc = 0; loc < size; loc++){
		if( array[loc] < array[min] ){
			min = loc;	
		}
	}
	return min;
}

//The Important True Function
void greedyClassic1( double *Y, double *initData, int *n, int *p, double *lambda,
			double *epsilon, int *M, int *correct, double *beta0 ,
									 double *betas, int *keep, int *numKept ){

	//vector size of beta, =1 if still in useable set, =0 if thrown away
	int use[*p];
	for( int j=0; j<*p; j++){
		use[j] = 1; 
	}

	//Set the data matrix (2D Array) from the initData
	int k=0; 
	double data[*n][*p];
	for( int j=0; j < *p; j++){
		for( int i=0; i < *n; i++){
			data[i][j] = initData[k];
			k++;
		}
	}

	//initial values for N+ and N-
	int N_plus = 0;
	int N_minus = 0;
	
	for(int i=0; i < *n; i++){
		if( Y[i] == 1.0){ 
			N_plus++;
		}else if( Y[i] == -1.0){
			N_minus++;
		}
	}
	
	//set the first beta_0 estimate
	*beta0 = sgn( N_plus - N_minus );
	//*beta0 = 0.0; //just for testing  
	  
	//Vector of residuals
	double S[*M+1];
	
	S[0] = 0.0; 
	for(int i=0; i < *n; i++){
		S[0] = S[0] + plus(1 - Y[i]* *beta0); 
	}
	
	//S[0]=1.0;
	
	//the matrix c that is n by M
	double c[*n][*M+1];
	
	for(int i=0; i<*n; i++){
		c[i][0] = 1.0 - Y[i]* *beta0;
		//Rprintf("%f    ", c[i][0]); 
	}
	//Rprintf("\n \n"); 
	
	//Counting parameters for algorithm
	double F_plus, F_minus, T1, T2, T3, T4; 

	//vector of current beta estimates, changes with each loop
	double beta_hat[*p];	
	
//Now Start the loop to select the variables
int m; //declare m outside for loop so scope is outside loop  
for(m=0; m < *M; m++){
	//Rprintf("%d :\n", m);
	
	for( int j=0; j < *p; j++){
		//do not calculate if use = 0
		if( use[j] == 0 ){
			beta_hat[j] = 0.0; 
		}else{ //find beta_hat
	
		//calculate F.plus and F.minus
		F_plus=F_minus=T1=T2=T3=T4=0.0; //set =0 to start
		for(int i=0; i<*n; i++){
			if(Y[i]==1 && data[i][j]>0 && c[i][m]>0){
				T1 = T1 + data[i][j]; 
			}else if( Y[i]==1 && data[i][j]<0 && c[i][m]>=0){
				T2 = T2 + data[i][j];
			}else if( Y[i]==-1 && data[i][j]>0 && c[i][m]>=0){
				T3 = T3 + data[i][j]; 
			}else if( Y[i]==-1 && data[i][j]<0 && c[i][m]>0){
				T4 = T4 + data[i][j];
			}
		}
		F_plus = -T1 - T2 + T3 + T4 + *lambda;
		T1=T2=T3=T4=0.0; 
		//Calculate F.minus
		for(int i=0; i<*n; i++){
			if(Y[i]==1 && data[i][j]>0 && c[i][m]>=0){
				T1 = T1 + data[i][j]; 
			}else if( Y[i]==1 && data[i][j]<0 && c[i][m]>0){
				T2 = T2 + data[i][j];
			}else if( Y[i]==-1 && data[i][j]>0 && c[i][m]>0){
				T3 = T3 + data[i][j]; 
			}else if( Y[i]==-1 && data[i][j]<0 && c[i][m]>=0){
				T4 = T4 + data[i][j];
			}
		}
		F_minus = -T1 - T2 + T3 + T4 - *lambda;
		//Rprintf("F_0+=%f \t F_0-=%f \n", F_plus, F_minus); 
		//Optimization Step
		if( F_minus <= 0.0 && F_plus >= 0.0){
			//set =0 and remove from "useable" set
			beta_hat[j] = 0.0;			
			use[j] = 0; 
			
		}else if( F_minus > 0.0){
			
			//array to hold values of cY/X vector
			double cYX[*n]; 
			for(int i=0; i < *n; i++){
				//Rprintf("%f   ", c[i][m]); 
				cYX[i] = c[i][m]*Y[i]/data[i][j]; 
			}
			//Rprintf("\n \n"); 
			//count how many negative values
			//add 1 to include first value = 0 
			int num_neg = numNeg(cYX, *n)+1; 
		 	
			//make vector of just negative values, first value is 0
			double negVec[num_neg];  
			negVec[0] = 0.0; 
			int count = 1; 
			for( int i = 0; i < *n; i++){
				if(cYX[i] < 0.0){
					negVec[count] = cYX[i];
					count++; 
				}	
			}
			
			//sort the negVec vector descending
			qsort( negVec, num_neg, sizeof(double), cmp_desc); 
			
			//get only the unique values
			int numUnique=0; double curVal=0.0;
			  int is_unique[num_neg];
			  is_unique[0] = numUnique = 1; //set initial values 
			  for( int i=1; i<num_neg; i++){
			 	if( fabs( negVec[i] - curVal ) > 1e-10 ){
				  is_unique[i] = 1; //set to 1 if unique
				  numUnique++;
				  curVal = negVec[i];  
				}else{
				  is_unique[i] = 0; //set to 0 if not
				}
			  }
			//make vector of just unique values
			double xi[numUnique];
			count=0;
			for( int i=0; i<num_neg; i++){
				if( is_unique[i] == 1){
					xi[count] = negVec[i];
					count++;
					//Rprintf("%f  ", negVec[i]); 
				}
			}
			//Rprintf("\n ");
			
			//loop through the negVec vector finding the knot point
			double T = F_minus; 
			int l=0; 
			while( T > 0.0 ){
				l++;
				for( int i=0; i<*n; i++){
					if( cYX[i] == xi[l] ){
						T = T - fabs( data[i][j] ); 
					}	
				} 
			}
			//Set the beta.hat value
			beta_hat[j] = xi[l];
			
			//Rprintf("%f  ", beta_hat[j]);
			
		
		}else if( F_plus < 0.0 ){
			
			//array to hold values of cY/X vector
			double cYX[*n]; 
			for(int i=0; i < *n; i++){
				//Rprintf("%f   ", c[i][m]); 
				cYX[i] = c[i][m]*Y[i]/data[i][j]; 
			}
			//Rprintf("\n \n"); 
			//count how many positive values
			//add 1 to include first value = 0 
			int num_pos = numPos(cYX, *n)+1; 
			
			//make vector of just positive values, first value is 0
			double posVec[num_pos];  
			posVec[0] = 0.0;
			int count=1;  
			for( int i = 0; i < *n; i++){
			  if(cYX[i] > 0.0){
				posVec[count] = cYX[i];
				count++; 
			  }	
			}
			
			//sort the negVec vector descending
			qsort( posVec, num_pos, sizeof(double), cmp_ascd); 
			
			//get only the unique values
			int numUnique=0; double curVal=0.0;
			int is_unique[num_pos];
			is_unique[0] = numUnique = 1; //set initial values 
			for( int i=1; i<num_pos; i++){
			  if( fabs( posVec[i] - curVal ) > 1e-10 ){
				is_unique[i] = 1; //set to 1 if unique
				numUnique++;
				curVal = posVec[i];  
			  }else{
				is_unique[i] = 0; //set to 0 if not
			  }
			}
			//make vector of just unique values
			double xi[numUnique];
			count=0;
			for( int i=0; i<num_pos; i++){
				if( is_unique[i] == 1){
					xi[count] = posVec[i];
					count++;
					//Rprintf("%f  ", posVec[i]); 
				}
			}
			//Rprintf("\n "); 
			
			//loop through the posVec vector finding the knot point
			double T = F_plus; 
			int l=1;
			for(int i=0; i<*n; i++){
				if( cYX[i] == xi[l]){
					T = T + fabs( data[i][j]); 	
				}
			} 
			while( T < 0.0 ){
				l++;
				for( int i=0; i<*n; i++){
					if( cYX[i] == xi[l] ){
						T = T + fabs( data[i][j] ); 
					}	
				} 
			}
			//Set the beta.hat value
			beta_hat[j] = xi[l]; 
		}
	  }
	}
	
	//Determine which is the best predictor
	double resids[*p], dummysum; 
	for( int j1 = 0; j1 < *p; j1++){
		dummysum = 0.0; 	
		for( int i=0; i<*n; i++){
			dummysum = dummysum + plus(c[i][m]-Y[i]*data[i][j1]*beta_hat[j1]);
			//Rprintf("i=%d,m=%d:   ", i, m); 
			//Rprintf("%f   ", c[i][m]);
		}
		resids[j1] = dummysum;
		//Rprintf("%f   ", resids[j1]);
	}
	//Rprintf("\n \n"); 
	  int bestj = minLoc(resids, *p); 
	
	//store the best predictor value
	  keep[m] = bestj; 
	//betas[bestj] = betas[bestj] + beta_hat[bestj];
	  betas[m] = beta_hat[bestj];
	  S[m+1] = resids[bestj];
	
	//update c values
	for( int i=0; i<*n; i++ ){
		c[i][m+1] = c[i][m] - (Y[i]*data[i][bestj]*beta_hat[bestj]);
	}
	
	//see if we should exit the loop
	//Rprintf( "%f \t %f \n", S[m+1], S[m] ); 
	if( S[m] - S[m+1] < *epsilon){
		m=m+1;
		break; 
	}
}

//Done with loop so say how many predictors were kept
*numKept = m; 

if( *correct==1){
  //Calculate F_0plus and F_0minus
  double F_0plus, F_0minus;
  F_0plus = F_0minus = 0.0; 
  for( int i=0; i<*n; i++){
	if( Y[i]==-1 && c[i][m] >= 0 ){
		F_0plus++;	
	}else if( Y[i]==1 && c[i][m] >= 0 ){
		F_0minus--;
	}
	if( Y[i]==1 && c[i][m] > 0 ){
		F_0plus--;  
	}else if( Y[i]==-1 && c[i][m] > 0){
		F_0minus++; 
	}
  }
   
  //See where do the values of F_0 fall 
  //if( F_0minus <= 0.0 && F_0plus >= 0.0 )	do nothing

  if( F_0minus > 0.0 ){
  	//array to hold values of cY vector
	double cY[*n]; 
	for(int i=0; i < *n; i++){
		cY[i] = c[i][m]*Y[i]; 
	}
	//count how many negative values
	//add 1 to include first value = 0 
	int num_neg = numNeg(cY, *n)+1; 
			
	//make vector of just negative values, first value is 0
	double negVec[num_neg];  
	negVec[0] = 0.0; 
	int count = 1; 
	for( int i = 1; i < *n; i++){
		if(cY[i] < 0.0){
			negVec[count] = cY[i];
			count++; 
		}	
	}
			
	//sort the negVec vector descending
	qsort( negVec, num_neg, sizeof(double), cmp_desc); 
			
	//get only the unique values
	int numUnique=0; double curVal=0.0;
	int is_unique[num_neg];
	is_unique[0] = numUnique = 1; //set initial values 
	for( int i=1; i<num_neg; i++){
		if( fabs( negVec[i] - curVal ) > 1e-10 ){
			is_unique[i] = 1; //set to 1 if unique
			numUnique++;
			curVal = negVec[i];  
		}else{
			is_unique[i] = 0; //set to 0 if not
		}
	}
	//make vector of just unique values
	double xi[numUnique];
	count=0; 
	for( int i=0; i<num_neg; i++){
		if( is_unique[i] == 1){
			xi[count] = negVec[i];
			count++;
		}
	}
	
	double T = F_0minus; 
	int l=0; 
	while( T > 0.0 ){
		l++;
		for( int i=0; i<*n; i++){
			if( cY[i] == xi[l] ){
			T = T - 1.0; 
			}	
		} 
	}
	//Set the beta.hat value
	*beta0 = *beta0 + xi[l];
	
  }else if( F_0plus < 0.0){
  	//array to hold values of cY/X vector
	double cY[*n]; 
	for(int i=0; i < *n; i++){
		cY[i] = c[i][m]*Y[i]; 
	}
	//count how many negative values
	//add 1 to include first value = 0 
	int num_pos = numPos(cY, *n)+1; 
			
	//make vector of just negative values, first value is 0
	double posVec[num_pos];  
	posVec[0] = 0.0;
	int count=1;  
	for( int i = 0; i < *n; i++){
		if(cY[i] > 0.0){
			posVec[count] = cY[i];
			count++; 
		}	
	}
			
	//sort the negVec vector descending
	qsort( posVec, num_pos, sizeof(double), cmp_ascd); 
			
	//get only the unique values
	int numUnique=0; double curVal=0.0;
	int is_unique[num_pos];
	is_unique[0] = numUnique = 1; //set initial values 
	for( int i=1; i<num_pos; i++){
		if( fabs( posVec[i] - curVal ) > 1e-10 ){
			is_unique[i] = 1; //set to 1 if unique
			numUnique++;
			curVal = posVec[i];  
		}else{
			is_unique[i] = 0; //set to 0 if not
		}
	}
	//make vector of just unique values
	double xi[numUnique];
	count=0;
	for( int i=0; i<num_pos; i++){
		if( is_unique[i] == 1){
			xi[count] = posVec[i];
			count++;
		}
	}
			
	//loop through the negVec vector finding the knot point
	double T = F_0plus; 
	int l=0; 
	while( T < 0.0 ){
		l++;
		for( int i=0; i<*n; i++){
			if( cY[i] == xi[l] ){
				T = T + 1.0; 
			}	
		} 
	}
	//Set the beta.hat value
	*beta0 = *beta0 + xi[l]; 	
  }
}
}	
	
	
	
	